# Generated by ariadne-codegen
# Source: tools/graphql_codegen/utils/

__all__ = ["SERVER_FEATURES_QUERY_GQL"]

SERVER_FEATURES_QUERY_GQL = """
query ServerFeaturesQuery {
  serverInfo {
    features {
      name
      isEnabled
    }
  }
}
"""
